#include <windows.h>
#include <sqlext.h>
#include <stdio.h>
#include <stdlib.h>


static bool FetchAndDisplayCatalogList (HSTMT hstmt)
{
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	retcode = SQLExecDirect (hstmt, (unsigned char *)
		"SELECT Abbreviation, LongName FROM CatalogNames "
		"WHERE HasNoCatNumbers = No ORDER BY Abbreviation",
		SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		char	abbreviation[11], longName[101];
		SDWORD	cbAbbreviation, cbLongName;

		puts ("");
		puts ("<P>");
		puts ("<BR>");
		puts ("<BR>");
		puts ("<FORM ACTION=\"/Scripts/search_star.exe\" METHOD=\"GET\">");
		puts ("OR, search for a star by its exact catalog number (use a * to "
			"indicate a degrees sign):");
		puts ("<BR><SELECT NAME=\"Catalog\">");

		SQLBindCol(hstmt, 1, SQL_C_CHAR, abbreviation, 11, &cbAbbreviation);
		SQLBindCol(hstmt, 2, SQL_C_CHAR, longName, 101, &cbLongName);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			printf ("<OPTION VALUE=\"%s\"> %s", abbreviation, abbreviation);
			if (cbLongName != SQL_NULL_DATA  &&  !isspace(longName[0]))
				printf (" (%s)", longName);
			printf ("\n");
		}

		puts ("</SELECT>");
		puts ("<INPUT TYPE=\"text\" NAME=\"CatNo\">");
		puts ("<INPUT TYPE=\"Submit\">");
		puts ("</FORM>");
		return	true;
	}
	return	false;
}


int main()
{
	HENV	henv; 
	HDBC	hdbc; 
	HSTMT	hstmt;
	RETCODE	retcode;
	int		mainRetCode = -1;

	printf ("Content-type: text/html%c%c", 10, 10);
	puts ("<HTML>\n");
	puts ("<HEAD>");
	puts ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=windows-1252\">");
	puts ("<META NAME=\"Author\" CONTENT=\"Roger M. Wilcox\">");
	puts ("<META NAME=\"Description\" CONTENT=\"A database of"
		" more than 2000 nearby stars\">");
	puts ("<META NAME=\"KeyWords\" CONTENT=\"astronomy,interstellar,stars,"
		"nearby stars,stellar,science,science fiction,astrophysics\">");
	puts ("<META NAME=\"viewport\" CONTENT=\"width=device-width, initial-scale=1\">");
	puts ("<TITLE>The Internet Stellar Database</TITLE>\n</HEAD>\n");
	puts ("<BODY>");

	puts ("<CENTER><H4>The</H4></CENTER>");
	puts ("<H1><CENTER>Internet STELLAR DATABASE</CENTER></H1>");
	puts ("<P>");
	puts ("<BR>");
	puts ("All the dirt on your favorite stars!&nbsp; Well, on those nearby stars");
	puts ("within 75 light-years, anyway.&nbsp; (Plus some of the more well-known");
	puts ("\"name brand\" stars farther away.)&nbsp; And even then, I skipped a heck");
	puts ("of a lot of solitary red dwarfs for which half-decent data weren't");
	puts ("available.&nbsp; Oh, and this site should probably have a few more ");
	puts ("goodies added to it in various places.&nbsp; But don't let that stop you.");
	puts ("<HR>");
	puts ("<P>");
	puts ("<FORM ACTION=\"/Scripts/search_star.exe\" METHOD=\"GET\">");
	puts ("Search for a star by name (e.g. Omicron(2) Eridani, or The Dog Star):<BR>");
	puts ("<INPUT TYPE=\"text\" NAME=\"Name\">");
	puts ("<INPUT TYPE=\"Submit\">");
	puts ("</FORM>");

	retcode = SQLAllocEnv(&henv);
	if (retcode == SQL_SUCCESS)
	{
		retcode = SQLAllocConnect(henv, &hdbc);
		if (retcode == SQL_SUCCESS)
		{
			SQLSetConnectOption(hdbc, SQL_LOGIN_TIMEOUT, 2);
			SQLSetConnectOption(hdbc, SQL_ACCESS_MODE, SQL_MODE_READ_ONLY);

			// Note: for the following statement to succeed, the system
			// running this code MUST have a System DSN of "ISDB"
			// configured to use the MS Access 97 driver with a "Database"
			// of C:\ISDB\ISDB.MDB in the ODBC32 Control Panel applet.
			retcode = SQLConnect(hdbc,
				(unsigned char *) "ISDB", SQL_NTS,
				(unsigned char *) "", 0, (unsigned char *) "", 0);
			if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO)
			{ 
				retcode = SQLAllocStmt(hdbc, &hstmt);
				if (retcode == SQL_SUCCESS)
				{
					if (FetchAndDisplayCatalogList (hstmt))
						mainRetCode = 0;
					SQLFreeStmt(hstmt, SQL_DROP);
				}
				else
				{
					printf ("Internal error: could not allocate statement."
							"  Please try again.\n");
				}
				SQLDisconnect(hdbc); 
			}
			else
			{
				char szSqlState [20];
				char szErrorMsg [SQL_MAX_MESSAGE_LENGTH];
				SDWORD	fNativeError;
				SWORD	cbErrorMsg;

				printf ("Content-type: text/plain\n\n");
				printf ("Internal error: SQLConnect() failed!!\n");
				retcode = SQLError (henv, hdbc, SQL_NULL_HSTMT,
					(UCHAR *) szSqlState, &fNativeError, (UCHAR *) szErrorMsg,
					SQL_MAX_MESSAGE_LENGTH - 1, &cbErrorMsg);
				if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO)
				{
					printf ("SQLSTATE = '%s'\n", szSqlState);
					printf ("Error message = '%s'\n", szErrorMsg);
				}
			}
			SQLFreeConnect(hdbc);
		}
		else
		{
			printf ("Internal error: could not allocate connection."
					"  Please try again.\n");
		}
		SQLFreeEnv(henv);
	}
	else
	{
		printf ("Internal error: could not allocate environment."
				"  Please try again.\n");
	}

	puts ("");
	puts ("<P>");
	puts ("<BR>");
	puts ("<BR>");
	puts ("<FORM ACTION=\"/Scripts/find_neighbors.exe\" METHOD=\"GET\">");
	puts ("OR, search for all stars within");
	puts ("<INPUT TYPE=\"text\" NAME=\"ly\" VALUE=\"15\" SIZE=3>");
	puts ("light-years of galactic coordinates:<BR>");
	puts ("X=<INPUT TYPE=\"text\" NAME=\"X\" VALUE=\"0.0\" SIZE=4>,");
	puts ("Y=<INPUT TYPE=\"text\" NAME=\"Y\" VALUE=\"0.0\" SIZE=4>,");
	puts ("Z=<INPUT TYPE=\"text\" NAME=\"Z\" VALUE=\"0.0\" SIZE=4><BR>");
	puts ("with Right Ascension between:<BR>");
	puts ("<INPUT TYPE=\"text\" NAME=\"minRAHour\" VALUE=\"0\" SIZE=2>h ");
	puts ("<INPUT TYPE=\"text\" NAME=\"minRAMin\" VALUE=\"0\" SIZE=2>m ");
	puts ("<INPUT TYPE=\"text\" NAME=\"minRASec\" VALUE=\"0.0\" SIZE=4>s and ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxRAHour\" VALUE=\"23\" SIZE=2>h ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxRAMin\" VALUE=\"59\" SIZE=2>m ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxRASec\" VALUE=\"59.999\" SIZE=4>s<BR>");
	puts ("and with Declination between:<BR>");
	puts ("<INPUT TYPE=\"text\" NAME=\"minDecDeg\" VALUE=\"-89\" SIZE=2>\xB0 ");
	puts ("<INPUT TYPE=\"text\" NAME=\"minDecMin\" VALUE=\"59\" SIZE=2>' ");
	puts ("<INPUT TYPE=\"text\" NAME=\"minDecSec\" VALUE=\"59.999\" SIZE=4>\" and ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxDecDeg\" VALUE=\"+89\" SIZE=2>\xB0 ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxDecMin\" VALUE=\"59\" SIZE=2>' ");
	puts ("<INPUT TYPE=\"text\" NAME=\"maxDecSec\" VALUE=\"59.999\" SIZE=4>\"");
	puts ("<INPUT TYPE=\"Submit\">");
	puts ("</FORM>");

	puts ("");
	puts ("<P>");
	puts ("<BR>");
	puts ("<BR>");
	puts ("<A HREF=\"intro.html\">Quicky introduction to stellar astronomy</A><BR>");
	puts ("<A HREF=\"fields.html\">What the fields for each star mean</A><BR>");
	puts ("<A HREF=\"scale.html\">The scale of interstellar space</A><BR>");
	puts ("<A HREF=\"evolution.html\">How stars evolve over their lifetimes</A><BR>");
	puts ("<A HREF=\"naming.html\">How stars are named</A><BR>");
	puts ("<A HREF=\"submissions.html\">Guidelines for submissions</A><BR>");
	puts ("<A HREF=\"about.html\">About the Internet Stellar Database</A></BR>");
	puts ("<A HREF=\"roll-your-own.html\">"
		"How to Make Your Own Internet Stellar Database</A> (in case I drop dead tomorrow)</BR>");
	puts ("<A HREF=\"links.html\">Links to other webpages related to stellar astronomy</A>");

	puts ("");
	puts ("<P>");
	puts ("<BR>");
	puts ("<BR>");
	puts ("You are visitor number");
	puts ("<a href=\"http://home.qpt.com/counter.html\">");
	puts ("<img border=\"0\" src=\"http://www.ultimatecounter.com/count/count.cgi?95954\"></a>");
	// puts ("<B>000000</B>");
	puts ("to the Internet Stellar Database since 5-September-1999.  (Note that this");
	puts ("statistic increases every time the same visitor, including this site's");
	puts ("webmaster, loads this webpage from a different web browser instance; so this");
	puts ("number is larger than the number of different visitors.)");
	puts ("<P>");
	puts ("<HR>");
	puts ("Send comments about this webpage to");
	puts ("<a href=\"mailto:rogermw@ix.netcom.com?SUBJECT=ISDb\">rogermw@ix.netcom.com</a>.");

	puts ("");
	puts ("</BODY>");
	puts ("</HTML>");
	return	mainRetCode;
}
